/*
 * Copyright The University of Chicago
 *
 * All Rights Reserved.
 */
#ifndef GLOBUS_GOOGLE_TOKEN_H
#define GLOBUS_GOOGLE_TOKEN_H

/**
 * @file globus_google_token.h
 * Interface for the GridFTP Google OAuth Token Handling
 */

#include "globus_google_drive_common.h"
#include "globus_common.h"
#include <time.h>

/**
 * @brief OAuth Token Handling
 * @defgroup globus_google_token OAuth Token Handling
 * @details
 * This component implements a security token cache for the DSI. This includes
 * API functions for constructing security consent URI, caching access and
 * refresh tokens, and for refreshing access tokens. It also provides a CLI
 * program to manage the token cache.
 */

#ifdef __cplusplus
extern "C" {
#endif

/* @{ */
/** @ingroup globus_google_token */
/** Client ID of the DSI */
extern char                            *globus_i_google_client_id;
/** Client Secret of the DSI (not really treated as secret */
extern char                            *globus_i_google_client_secret;
/** OAuth 2 authentication URI */
extern const char                      *globus_i_google_auth_uri;
/** OAuth 2 token URI */
extern const char                      *globus_i_google_token_uri;
/** OAuth 2 requested scopes: these include the drive access scope
 * and the appdata scope to store private data */
extern const char                      *globus_i_google_scope;
/** OAuth 2 redirect uri */
extern const char                      *globus_i_google_redirect_uri;
/** Default token cache */
extern const char                      *globus_i_google_token_default_cache;

/**
 * @brief Access token json field name
 */
#define GLOBUS_GOOGLE_TOKEN_ACCESS_TOKEN "access_token"
/**
 * @brief Access token expiration json field name
 */
#define GLOBUS_GOOGLE_TOKEN_ACCESS_TOKEN_EXPIRATION "access_token_expiration"
/**
 * @brief Refresh token json field name
 */
#define GLOBUS_GOOGLE_TOKEN_REFRESH_TOKEN "refresh_token"
/* @} */

globus_result_t
globus_google_token_request_auth(
    char                              **auth_uri);

globus_result_t
globus_google_token_init(
    const char                         *auth_code,
    char                              **refresh_tokenp,
    char                              **access_tokenp,
    time_t                             *access_token_expirationp);

globus_result_t
globus_google_token_refresh(
    char                               *refresh_token,
    char                              **access_token,
    time_t                             *access_token_expiration);

globus_result_t
globus_google_token_path_get(
    char                              **token_cache_path);

globus_result_t
globus_google_token_load(
    const char                         *token_cache_path,
    const char                         *initial_token_data,
    char                              **refresh_token,
    char                              **access_token,
    time_t                             *access_token_expiration);

globus_result_t
globus_google_token_save(
    const char                         *token_cache_path,
    const char                         *refresh_token,
    const char                         *access_token,
    time_t                              access_token_expiration);

globus_result_t
globus_i_google_token_response_check(
    const char                         *method,
    const char                         *uri,
    globus_dsi_rest_response_arg_t     *response,
    json_t                             *json_error);

extern globus_module_descriptor_t       globus_i_google_token_module;

/**
 * @brief Module descriptor for the token management API
 * @ingroup globus_google_token
 * @hideinitializer
 */
#define GLOBUS_GOOGLE_TOKEN_MODULE (&globus_i_google_token_module)

#ifdef __cplusplus
}
#endif

#endif /* GLOBUS_GOOGLE_TOKEN_H */
